@@ -53,6 +53,7 @@ var (
 	oldMetadata, newMetadata *metadata.Descriptor
 	osInfo                   osinfo.OSInfo
 	mdsClient                *metadata.Client
+	addressManager           = &addressMgr{}
 )
 
 const (
@@ -86,7 +87,7 @@ func closeFile(c io.Closer) {
 
 func availableManagers() []manager {
 	managers := []manager{
-		&addressMgr{},
+		addressManager,
 	}
 
 	if runtime.GOOS == "windows" {
@@ -104,45 +105,48 @@ func availableManagers() []manager {
 	)
 }
 
+func runManager(ctx context.Context, mgr manager) {
+	disabled, err := mgr.Disabled(ctx)
+	if err != nil {
+		logger.Errorf("Failed to run manager's Disabled() call: %+v", err)
+		return
+	}
+
+	if disabled {
+		logger.Debugf("manager %#v disabled, skipping", mgr)
+		return
+	}
+
+	timeout, err := mgr.Timeout(ctx)
+	if err != nil {
+		logger.Errorf("[%#v] Failed to run manager Timeout() call: %+v", mgr, err)
+		return
+	}
+
+	diff, err := mgr.Diff(ctx)
+	if err != nil {
+		logger.Errorf("[%#v] Failed to run manager Diff() call: %+v", mgr, err)
+		return
+	}
+
+	if !timeout && !diff {
+		logger.Debugf("[%#v] Manager reports no diff", mgr)
+		return
+	}
+
+	logger.Debugf("running %#v manager", mgr)
+	if err := mgr.Set(ctx); err != nil {
+		logger.Errorf("[%#v] Failed to run manager Set() call: %s", mgr, err)
+	}
+}
+
 func runUpdate(ctx context.Context) {
 	var wg sync.WaitGroup
 	for _, mgr := range availableManagers() {
 		wg.Add(1)
 		go func(mgr manager) {
 			defer wg.Done()
-
-			disabled, err := mgr.Disabled(ctx)
-			if err != nil {
-				logger.Errorf("Failed to run manager's Disabled() call: %+v", err)
-				return
-			}
-
-			if disabled {
-				logger.Debugf("manager %#v disabled, skipping", mgr)
-				return
-			}
-
-			timeout, err := mgr.Timeout(ctx)
-			if err != nil {
-				logger.Errorf("[%#v] Failed to run manager Timeout() call: %+v", mgr, err)
-				return
-			}
-
-			diff, err := mgr.Diff(ctx)
-			if err != nil {
-				logger.Errorf("[%#v] Failed to run manager Diff() call: %+v", mgr, err)
-				return
-			}
-
-			if !timeout && !diff {
-				logger.Debugf("[%#v] Manager reports no diff", mgr)
-				return
-			}
-
-			logger.Debugf("running %#v manager", mgr)
-			if err := mgr.Set(ctx); err != nil {
-				logger.Errorf("[%#v] Failed to run manager Set() call: %s", mgr, err)
-			}
+			runManager(ctx, mgr)
 		}(mgr)
 	}
 	wg.Wait()
@@ -150,6 +154,11 @@ func runUpdate(ctx context.Context) {
 
 func runAgent(ctx context.Context) {
 	opts := logger.LogOpts{LoggerName: programName}
+
+	if !cfg.Get().Core.CloudLoggingEnabled {
+		opts.DisableCloudLogging = true
+	}
+
 	if runtime.GOOS == "windows" {
 		opts.FormatFunction = logFormatWindows
 		opts.Writers = []io.Writer{&utils.SerialPort{Port: "COM1"}}
@@ -169,6 +178,9 @@ func runAgent(ctx context.Context) {
 		os.Exit(1)
 	}
 
+	// Try flushing logs before exiting, if not flushed logs could go missing.
+	defer logger.Close()
+
 	logger.Infof("GCE Agent Started (version %s)", version)
 
 	osInfo = osinfo.Get()
@@ -220,7 +232,7 @@ func runAgent(ctx context.Context) {
 		logger.Debugf("Handling metadata %q event.", evType)
 
 		// If metadata watcher failed there isn't much we can do, just ignore the event and
-		// allow the water to get it corrected.
+		// allow the watcher to get it corrected.
 		if evData.Error != nil {
 			logger.Infof("Metadata event watcher failed, ignoring: %+v", evData.Error)
 			return true
