@@ -1,23 +1,25 @@
-//  Copyright 2017 Google Inc. All Rights Reserved.
-//
-//  Licensed under the Apache License, Version 2.0 (the "License");
-//  you may not use this file except in compliance with the License.
-//  You may obtain a copy of the License at
-//
-//      http://www.apache.org/licenses/LICENSE-2.0
-//
-//  Unless required by applicable law or agreed to in writing, software
-//  distributed under the License is distributed on an "AS IS" BASIS,
-//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-//  See the License for the specific language governing permissions and
-//  limitations under the License.
+// Copyright 2017 Google LLC
+
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+
+//     https://www.apache.org/licenses/LICENSE-2.0
+
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 package main
 
 import (
-	"os/exec"
+	"context"
+	"fmt"
 	"strings"
 
+	"github.com/GoogleCloudPlatform/guest-agent/google_guest_agent/run"
 	"github.com/GoogleCloudPlatform/guest-logging-go/logger"
 	"golang.org/x/sys/windows/registry"
 )
@@ -105,9 +107,9 @@ func deleteRegKey(key, name string) error {
 	return k.DeleteValue(name)
 }
 
-func checkWindowsServiceRunning(servicename string) bool {
-	res := runCmdOutput(exec.Command("sc", "query", servicename))
-	return strings.Contains(res.Stdout(), "RUNNING")
+func checkWindowsServiceRunning(ctx context.Context, servicename string) bool {
+	res := run.WithOutput(ctx, "sc", "query", servicename)
+	return strings.Contains(res.StdOut, "RUNNING")
 }
 
 func getWindowsServiceImagePath(regKey string) (string, error) {
@@ -119,11 +121,11 @@ func getWindowsServiceImagePath(regKey string) (string, error) {
 	return imagePath, nil
 }
 
-func getWindowsExeVersion(path string) (versionInfo, error) {
+func getWindowsExeVersion(ctx context.Context, path string) (versionInfo, error) {
 	psCmd := "(Get-Item '" + path + "').VersionInfo.FileVersion"
-	psVer, err := exec.Command("powershell", "-c", psCmd).Output()
-	if err != nil {
-		return versionInfo{0, 0}, err
+	res := run.WithOutput(ctx, "powershell", "-c", psCmd)
+	if res.ExitCode != 0 {
+		return versionInfo{0, 0}, fmt.Errorf(res.Error())
 	}
-	return parseVersionInfo(psVer)
+	return parseVersionInfo([]byte(res.StdOut))
 }
